
const API_BASE_TOKEN_URL = "https://prod.api.tigo.com/v1/";
const API_BASE_PAYMENT_URL = "https://prod.api.tigo.com/v2/";
const SANDBOX_API_BASE_TOKEN_URL = "https://securesandbox.tigo.com/v1/";
const SANDBOX_API_BASE_PAYMENT_URL = "https://securesandbox.tigo.com/v2/";
let options;
let phone;

function setData(data) {
  const colect = new Map(Object.entries(data));
  options = colect;
  createForm();
}

function getPath() {
  const path = options.get("basePath");
  return path;
}

function getBaseTokenURL() {
  if (options.get("sandbox") == true) {
    return SANDBOX_API_BASE_TOKEN_URL;
  } else {
    return API_BASE_TOKEN_URL;
  }
}

function getBasePaymentURL() {
  if (options.get("sandbox") == true) {
    return SANDBOX_API_BASE_PAYMENT_URL;
  } else {
    return API_BASE_PAYMENT_URL;
  }
}


function createForm() {
  const form = document.getElementById("edit-payment-process-offsite-payment");
  let html = `
  <div class="modal fade tigo-modal" id="modalTigo" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" aria-labelledby="staticBackdropLabel" aria-hidden="true">
    <div class="modal-dialog">
      <div class="modal-content">
        <div class="modal-header">
          <img src="/${getPath()}/images/tigo_money.png" alt="Tigo Money" style="width:auto;height:auto;">
          <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
        </div>
        <div class="modal-body">
        <label for="inputTitle" class="form-label">Número de teléfono Tigo</label>
        <div class="row g-3 align-items-center">
          <div class="col-auto">
          <img src="/${getPath()}/images/PY.png" alt="PY" style="width:auto;height:25px;">
          </div>
          <div class="col-auto">
            <input type="number" id="inputNumber" onfocus="this.value=''" class="form-control" aria-describedby="numberHelpBlock">
          </div>
          <div class="col-auto">
            <div class="spinner-border spinner-border-sm visually-hidden" role="status" id="loading">
             <span class="visually-hidden">Loading...</span>
            </div>
          </div>
        </div>
        <div id="numberHelpBlock" class="form-text">
          Ingresa un número de teléfono válido para la transacción comensando con 09.
        </div>
        <div id="errorBlock" class="form-text" role="alert">
           <p class="text-danger" id='errorText'></p>
        </div>
        </div>
        <div class="modal-footer">
          <button type="button" class="btn btn-primary" onclick="checkForm();" id="send">Enviar</button>
          <button type="button" class="btn btn-secondary" data-bs-dismiss="modal">Cancelar</button>
        </div>
      </div>
    </div>
  </div>
    `;
  form.insertAdjacentHTML("afterend", html);
}

function checkForm() {
  var input = document.getElementById("inputNumber").value;
  var numberMessage = checkNumber(input);
  if (numberMessage) {
    document.getElementById('errorText').innerText = numberMessage;
  } else {
    actionEnable();
    phone = input;
    document.getElementById('errorText').innerText = "";
    authorization();
  }
}

function checkNumber(number){
  let numberData = number;
  let res = numberData.substring(0, 2);
  let count = numberData.length;

  if (count !== 10)
      return 'Número debe ser de 10 dígitos.';

  if (res !== '09')
      return 'Número debe empezar por 09.';

  return '';

}

async function authorization() {
  const apiToken = await getToken().catch((error => {
    console.error(error);
  }));
    try {
      const response = await axios.post(getBasePaymentURL() + 'tigo/mfs/payments/authorizations',{
        MasterMerchant : {
          account: options.get("account"),
          pin: options.get("pin"),
          id: options.get("name")
        },
        Subscriber: {
          account: phone,
          countryCode: '595',
          country: 'PRY',
          emailId: ''
        },
        redirectUri: options.get("redirect"),
        callbackUri: '',
        language: 'spa',
        OriginPayment: {
          amount: options.get("amount"),
          currencyCode: 'PYG',
          tax: '0.00',
          fee: '0.00'
        },
        exchangeRate: '1',
        LocalPayment : {
          amount: options.get("amount"),
          currencyCode: 'PYG'
        },
        merchantTransactionId: options.get("merchantTransactionId")
      },
      {
        headers: {
          'Content-Type': 'application/json',
           Authorization: 'Bearer ' + apiToken,
        }
      });

      return window.location.replace = response.data.redirectUrl;

    } catch (error) {
      actionDisable();
      console.error(error);
    }
}

async function getToken() {
  try {
    const response = await axios.post(getBaseTokenURL() + 'oauth/mfs/payments/tokens', {
       grant_type: 'client_credentials'
    },
      {
        headers: {
          'Content-Type': 'application/x-www-form-urlencoded',
          Authorization: 'Basic ' + options.get("credentials")
        }
    });

    return response.data.access_token;
    
  } catch (error) {
    actionDisable();
    console.error(error);
  }
}

function actionDisable() {
  document.getElementById("send").disabled = false;
  document.getElementById("loading").classList.add("visually-hidden");
}

function actionEnable() {
  document.getElementById("send").disabled = true;
  document.getElementById("loading").classList.remove("visually-hidden");
}