<?php

/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
namespace Drupal\commerce_tigo_money\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\TransferException;
use Symfony\Component\HttpFoundation\Request;
/**
 * @CommercePaymentGateway(
 *   id = "tigo_money_redirect_checkout",
 *   label = @Translation("Tigo Money OffSite."),
 *   display_label = @Translation("Tigo Money"),
 *   forms = {
 *    "offsite-payment" = "Drupal\commerce_tigo_money\PluginForm\TigoMoneyOffsite\PaymentForm",
 * },
 * modes= {
 *     "test" = "Test",
 *     "live" = "Live"
 * }
 * )
 */

class TigoMoneyPaymentGateway extends OffsitePaymentGatewayBase implements TigoMoneyPaymentGatewayInterface {
  
  /**
   * {@inheritdoc}
   */
  public function getPublicKey() {
    return $this->configuration['api_key'];
  }

  /**
   * {@inheritdoc}
   */
  public function getSecretKey() {
    return $this->configuration['api_secret'];
  }

  /**
   * {@inheritdoc}
   */
  public function getStoreName() {
    return $this->configuration['account_name'];
  }

  /**
   * {@inheritdoc}
   */
  public function getAccount() {
    return $this->configuration['account'];
  }

  /**
   * {@inheritdoc}
   */
  public function getPin() {
    return $this->configuration['pin'];
  }
  
  /**
   * @inheritdoc
   */
  public function defaultConfiguration() {
    return [
      'account_name' => 'Amazon',
      'account' => '0981123456',
      'pin' => '1234',
      'api_key' => 'yFqqXJh9PqnugfrsdfffDff3sf556',
      'api_secret' => '0808sEWdffASvvdd355sskl',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state); // TODO: Change the autogenerated stub
    
    $form['account_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Agent Store Name'),
      '#default_value' => $this->configuration['account_name'],
      '#description' => $this->t('Your store name.'),
      '#required' => TRUE,
    ];
    
    $form['account'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Agent Account'),
      '#default_value' => $this->configuration['account'],
      '#description' => $this->t('Your tigo money phone number acredited.'),
      '#required' => TRUE,
    ];
    $form['pin'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Agent PIN'),
      '#default_value' => $this->configuration['pin'],
      '#description' => $this->t('Your tigo money agent pin.'),
      '#required' => TRUE,
    ];
    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Api key'),
      '#default_value' => $this->configuration['api_key'],
      '#description' => $this->t('Unique client identifier assigned during the registration process with Tigo Money.'),
      '#required' => TRUE,
    ];
    $form['api_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Secret key'),
      '#default_value' => $this->configuration['api_secret'],
      '#description' => $this->t('Secret password provided during the registration process with Tigo Money.'),
      '#required' => TRUE,
    ];

    return $form;
  }
  
  /**
   * {@inheritdoc}
   */

  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state); // TODO: Change the autogenerated stub

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['account_name'] = $values['account_name'];
      $this->configuration['account'] = $values['account'];
      $this->configuration['pin'] = $values['pin'];
      $this->configuration['api_key'] = $values['api_key'];
      $this->configuration['api_secret'] = $values['api_secret'];
    }
  }

   /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $logger = \Drupal::logger('commerce_tigo_money');

    $response = urldecode($request->query->get('resp'));

    $logger->info('Tigo Money returned: ' . $response);

    $response = json_decode($response, TRUE);

    $transactionStatus = urldecode($request->query->get('transactionStatus'));
    $merchantTransactionReference = urldecode($request->query->get('merchantTransactionId'));
    $tigoTransactionReference = urldecode($request->query->get('mfsTransactionId'));
    
    if ($transactionStatus == 'success') {
      $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
      
      $payment = $payment_storage->create([
        'state' => 'completed',
        'amount' => $order->getTotalPrice(),
        'payment_gateway' => $this->parentEntity->id(),
        'order_id' => $order->id(),
        'remote_id' => $tigoTransactionReference,
        'remote_state' => $transactionStatus,
      ]);
      
      $logger->info('Saving Payment information. Transaction reference: ' . $merchantTransactionReference);
      $payment->save();
      \Drupal::messenger()->addStatus('Payment was processed');

      $logger->info('Payment information saved successfully. Transaction reference: ' . $merchantTransactionReference);

    } else {
      $logger->warning('Error payment');
      throw new PaymentGatewayException('Error payment is not processed.');
    }
  
  }
}
