<?php

/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
namespace Drupal\commerce_tigo_money\PluginForm\TigoMoneyOffsite;

use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Url;
use Drupal\commerce_tigo_money\ApiClient;
use Drupal\Core\StringTranslation\StringTranslationTrait;

class PaymentForm extends BasePaymentOffsiteForm {
  
  use StringTranslationTrait;

  protected $integrityHash;
  
  /**
   *    * {@inheritdoc}
   * */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['#attached']['library'][] = 'commerce_tigo_money/axios';
    $form['#attached']['library'][] = 'commerce_tigo_money/form';
    $form['#attached']['library'][] = 'commerce_tigo_money/checkout';

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    /** @var \Drupal\commerce_tigo_money\Plugin\Commerce\PaymentGateway\TigoMoneyPaymentGatewayInterface $plugin */
    $plugin = $payment->getPaymentGateway()->getPlugin();

    $clientId = $plugin->getPublicKey();
    $clientSecret = $plugin->getSecretKey();
    $agentAccount = $plugin->getAccount();
    $agentPin = $plugin->getPin();
    $agentName = $plugin->getStoreName();
    $gateway_mode = $plugin->getMode();
    
    if ($gateway_mode == 'live') {
      $sandbox = false;
    } else {
      $sandbox = true;
    }

    // Payment data.
    $payment_amount = $payment->getAmount()->getNumber();
    $order_id = $payment->getOrderId();
    $currency_code = $payment->getAmount()->getCurrencyCode();
    $order = $payment->getOrder();
    
    $path_mod = $themePath = \Drupal::service('extension.path.resolver')->getPath('module', 'commerce_tigo_money');

    $data = [
      'account' => $agentAccount,
      'pin' => $agentPin,
      'name' => $agentName,
      'amount' => $payment_amount,
      'currencyCode' => $currency_code,
      'merchantTransactionId' => $order_id,
      'credentials' => $this->encodeCredentials(),
      'redirect' => $form['#return_url'],
      'sandbox' => $sandbox,
      'basePath' => $path_mod
    ];

    $form = $this->buildRedirectForm($form, $form_state, '', $data, '');
    //$this->calculateChecksum($data);
    //$data = array_merge($data, ['integrity_hash' => $this->integrityHash]);
    $form['#attached']['drupalSettings']['transactionData'] = $data; //json_encode($data);

    return $form;

  }

    /**
   * {@inheritdoc}
   */
  public function buildRedirectForm(array $form, FormStateInterface $form_state, $redirect_url, array $data, $redirect_method = BasePaymentOffsiteForm::REDIRECT_POST) {
    $helpMessage = $this->t('Enter your phone number in the payment window, after sending you will receive an sms with a code that you must enter on the screen of your phone, then enter the PIN code from your wallet to confirm the transaction.');
    $form['commerce_message'] = [
      '#markup' => '<div class="checkout-help">' . $helpMessage . '</div>',
      '#process' => [
        [get_class($this), 'processRedirectForm'],
      ],
    ];

    return $form;
  }

   /**
   * {@inheritdoc}
   */
  public static function processRedirectForm(array $element, FormStateInterface $form_state, array &$complete_form) {
    $complete_form['#attributes']['class'][] = 'payment-redirect-form';
    unset($element['#action']);
    // The form actions are hidden by default, but needed in this case.
    $complete_form['actions']['#access'] = TRUE;

    foreach (Element::children($complete_form['actions']) as $element_name) {
      $complete_form['actions'][$element_name]['#access'] = TRUE;
    }
    return $element;
  }

  public function encodeCredentials() {
    /** @var \Drupal\commerce_tigo_money\Plugin\Commerce\PaymentGateway\TigoMoneyPaymentGatewayInterface $plugin */
    $plugin = $this->plugin;
    $access = $plugin->getPublicKey() . ":" . $plugin->getSecretKey();
    
    return base64_encode($access);
  }

  protected function calculateChecksum(array $options) {
    ksort($options);

    $hashedPayload = '';

    foreach ($options as $key => $value) {
      $hashedPayload = $value;
    }

    /** @var \Drupal\commerce_tigo_money\Plugin\Commerce\PaymentGateway\TigoMoneyPaymentGatewayInterface $plugin */
    $plugin = $this->plugin;

    $completeHash = $hashedPayload . $plugin->getSecretKey();
    $hash = hash('sha256', $completeHash);

    $this->integrityHash = $hash;
  }
}